require(["../../webjars/leaflet/1.3.1/leaflet.js", 'jquery'], function(leaflet, $) {
  'use strict';

  XWiki.leafletMap = Class.create({

    initialize: function(element, options) {
      this.element = $(element);
      // Default option values
      this.options = (options !== undefined && typeof options == 'object' ) ? options : {};
      var zoom = this.options.zoom !== undefined ? this.options.zoom : 14;
      //this.options.location       = this.options.location !== undefined       ? this.options.location.split(','): [48.864716, 2.349014];
      var mapId = this.options.mapId !== undefined ? this.options.mapId : 'map-id';
      var tiles = this.options.tiles;
      var locationType = this.options.locationType ? this.options.locationType.toLowerCase() : 'address';
      var info = this.options.info;
      var that = this;
      if (locationType == 'latlng') {
        var latLng = this.options.location;
        this.createMap(mapId, tiles, zoom, latLng, info);
      } else {
        var queryUrl = "https://nominatim.openstreetmap.org/search";
        var q = this.options.location;
        $.getJSON(queryUrl, {'q':q, 'format':'json'}).then(function(data) {
          if (data != undefined && data.length > 0) {
            var latLng = [data[0].lat, data[0].lon];
            that.createMap(mapId, tiles, zoom, latLng, info);
          } else {
            new XWiki.widgets.Notification('Error while geocoding the address "' + q +'" with OpenStreetMap Nominatim', 'error');
          }
        });
      }
    },

    createMap: function(mapId, tiles, zoom, latLng, info) {
      var mymap = L.map(mapId).setView(latLng, zoom);
      L.tileLayer(tiles, {
        attribution: '&copy; <a href="https://www.openstreetmap.org/">OpenStreetMap</a>',
        maxZoom: 18,
        id: 'mapbox.streets'
      }).addTo(mymap);

      var icon = L.icon({
        iconUrl: "../../webjars/leaflet/1.3.1/images/marker-icon.png",
        iconRetinaUrl: "../../webjars/leaflet/1.3.1/images/marker-icon-2x.png",
        shadowUrl: "../../webjars/leaflet/1.3.1/images/marker-shadow.png",
        iconSize: [25,41],
        iconAnchor: [12,41],
        popupAnchor: [1,-34],
        tooltipAnchor: [16,-28],
        shadowSize: [41,41]
      });

      var marker = L.marker(latLng, {icon: icon}).addTo(mymap);
      marker.on('click', onClick);
      if (info !== undefined) {
        marker.bindPopup(info);
      }

      function onClick() {
        if (info === undefined) {
          var url = "https://www.openstreetmap.org/directions?engine=graphhopper_foot&to="+ latLng+ "#map=19/"+ latLng[0]+"/"+ latLng[1];
          window.open(url,'_blank');
        }
      }
    }
  });

  var init = function(event, data) {
    var container = $((data && data.elements) || document);
    container.find('.xwiki-map-container').each(function() {
      var item = this;
      var library = item.dataset.library;
      if (library == "leaflet") {
        var options = {};
        var mapId = item.id;
        if (item.textContent) {
          options.info = item.textContent;
        }
        if (mapId) {
          options.mapId = mapId;
        }
        var zoom = item.dataset.zoom;
        if (zoom) {
          options.zoom = zoom;
        }
        var location = item.dataset.location;
        if (location) {
          options.location = location;
        }
        var locationType = item.dataset.locationType;
        if (locationType) {
          options.locationType = locationType;
        }
        else {
          options.locationType = 'address';
        }
        var tiles = item.dataset.tiles;
        if (tiles) {
          options.tiles = tiles;
        }
        else {
          options.tiles = "https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png";
        }
        new XWiki.leafletMap(item, options);
      }
    });
  };

  $(document).on('xwiki:dom:updated', init);
  $(init);
});
